#!/bin/bash

ZLLDB=""
MAINUSER=$(getent passwd 1000 | cut -d: -f1)
DECONZ_CONF_DIR="/home/$MAINUSER/.local/share"

TIMEOUT=0      # main loop iteration timeout, can be controlled by SIGUSR1
LOG_LEVEL=4

LOG_EMERG=
LOG_ALERT=
LOG_CRIT=
LOG_ERROR=
LOG_WARN=
LOG_NOTICE=
LOG_INFO=
LOG_DEBUG=

[[ $LOG_LEVEL -ge 0 ]] && LOG_EMERG="<0>"
[[ $LOG_LEVEL -ge 1 ]] && LOG_ALERT="<1>"
[[ $LOG_LEVEL -ge 2 ]] && LOG_CRIT="<2>"
[[ $LOG_LEVEL -ge 3 ]] && LOG_ERROR="<3>"
[[ $LOG_LEVEL -ge 4 ]] && LOG_WARN="<4>"
[[ $LOG_LEVEL -ge 5 ]] && LOG_NOTICE="<5>"
[[ $LOG_LEVEL -ge 6 ]] && LOG_INFO="<6>"
[[ $LOG_LEVEL -ge 7 ]] && LOG_DEBUG="<7>"

function init {
    # is sqlite3 installed?
	sqlite3 --version &> /dev/null
	if [ $? -ne 0 ]; then
		[[ $LOG_WARN ]] && echo "${LOG_WARN}sqlite3 not installed"
		return
	fi

	# look for latest config in specific order
	drs=("data/dresden-elektronik/deCONZ/zll.db" "dresden-elektronik/deCONZ/zll.db" "deCONZ/zll.db")
	for i in "${drs[@]}"; do
		if [ -f "${DECONZ_CONF_DIR}/$i" ]; then
			ZLLDB="${DECONZ_CONF_DIR}/$i"
			break
		fi
	done

	if [ ! -f "$ZLLDB" ]; then
		# might have been deleted
		ZLLDB=""
	fi

	if [[ -z "$ZLLDB" ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}no database file (zll.db) found"
		return
	fi
}

function checkHomebridge {

	local PROXY_ADDRESS=""
	local PROXY_PORT=""
	local BRIDGEID=""
	local HOMEBRIDGE=""

	## get database config
	params=( [0]="bridgeid" [1]="homebridge" [2]="proxyaddress" [3]="proxyport")
	values=()

	for i in {0..3}; do
		param=${params[$i]}
		value=$(sqlite3 $ZLLDB "select * from config2 where key=\"${param}\"")
		if [ $? -ne 0 ]; then
			return
		fi

		value=$(echo $value | cut -d'|' -f2)

		# basic check for non empty
		if [[ ! -z "$value" ]]; then
			values[$i]=$(echo $value | cut -d'|' -f2)
		fi
	done

	## all parameters found and valid?
	if [ -z "${values[0]}" ]; then # TODO: add values[1] when implementation in phoscon app comes
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}missing parameters"
		return
	fi

	BRIDGEID="${values[0]}"
	HOMEBRIDGE="${values[1]}" # disabled | managed | not-managed | installing
	PROXY_ADDRESS="${values[2]}"
	PROXY_PORT="${values[3]}"

	## install homebridge if not installed
	which homebridge
	if [ $? -ne 0 ]; then
		[[ $LOG_INFO ]] && echo "${LOG_INFO}installing homebridge plugin"

		curl --head --connect-timeout 20 -k https://www.phoscon.de &> /dev/null
		if [ $? -ne 0 ]; then
			if [[ ! -z "$PROXY_ADDRESS" && ! -z "$PROXY_PORT" && "$PROXY_ADDRESS" != "none" ]]; then
				export https_proxy="http://${PROXY_ADDRESS}:${PROXY_PORT}"
				export http_proxy="http://${PROXY_ADDRESS}:${PROXY_PORT}"
				[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}set proxy: ${PROXY_ADDRESS}:${PROXY_PORT}"
			else
				[[ $LOG_WARN ]] && echo "${LOG_WARN}no internet connection. Abort homebridge installation."
				return
			fi
		fi

		curl -sL https://deb.nodesource.com/setup_8.x | bash -
		apt-get install -y nodejs
		npm -g install npm@latest
		npm -g install homebridge --unsafe-perm
		npm -g install homebridge-hue
	fi

	# TODO: will be implemented when integration in phoscon app comes.
	# homebridge management will be done by phoscon app.
	# now its just enabling/ disabling this service by cmdline
	#case "$HOMEBRIDGE" in
    #    "disabled")
    #        # TODO
    #        ;;
    #
    #    "managed")
    #        # TODO
    #        ;;
    #
    #    "not-managed")
    #        # TODO
    #        ;;
	#
	#    "installing")
    #        # TODO
    #        ;;
	#
    #    *)
    #        [[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}invalid value $HOMEBRIDGE for parameter 'homebridge'"
    #        return
	#esac

	## create apikey for homebridge apps	
	local HOMEBRIDGE_AUTH=$(sqlite3 $ZLLDB "select * from auth where devicetype='Homebridge-hue'")
	if [[ $HOMEBRIDGE_AUTH == "" ]]; then
		sqlite3 $ZLLDB "insert into auth (apikey, devicetype, createdate) values (hex(randomblob(8)), 'Homebridge-hue', strftime('%Y-%m-%dT%H:%M:%S', 'now'))"				
	fi

	## configure config.json only if no configuration already exists

	if [[ ! -d /home/$MAINUSER/.homebridge ]]; then
		mkdir /home/$MAINUSER/.homebridge
	fi

	if [[ -f /home/$MAINUSER/.homebridge/config.json ]]; then
		[[ $LOG_DEBUG ]] && echo "${LOG_DEBUG}found existing homebridge config.json"
	else
		local APIKEY=$(sqlite3 $ZLLDB "select apikey from auth where devicetype='Homebridge-hue'")	
		echo "{
    \"bridge\": {
        \"name\": \"Phoscon Homebridge\",
        \"username\": \"00:21:2E:FF:FF:00\",
        \"port\": 51826,
        \"pin\": \"111-11-111\"
    },       

    \"platforms\": [
    {
      \"platform\": \"Hue\",
      \"users\": {
        \"$BRIDGEID\": \"$APIKEY\"
            },
      \"sensors\": true,
      \"nativeHomeKitSensors\": false,
      \"excludeSensorTypes\": [\"CLIPPresence\", \"Geofence\"],
      \"lights\": true,
	  \"hueMotionTemperatureHistory\": true
    }
  ]
}" > /home/$MAINUSER/.homebridge/config.json
	fi
	
	## daemonise plugin and enable autostart
	if [[ ! -f /etc/default/homebridge ]]; then
		echo "# Defaults / Configuration options for homebridge
# The following settings tells homebridge where to find the config.json file and where to persist the data (i.e. pairing and others)
HOMEBRIDGE_OPTS=-U /var/homebridge

# If you uncomment the following line, homebridge will log more 
# You can display this via systemd's journalctl: journalctl -f -u homebridge
# DEBUG=*" > /etc/default/homebridge
	fi

	if [[ ! -f /etc/systemd/system/homebridge.service ]]; then
		echo "[Unit]
Description=Node.js HomeKit Server 
After=syslog.target network-online.target

[Service]
Type=simple
User=homebridge
EnvironmentFile=/etc/default/homebridge
ExecStart=/usr/bin/homebridge $HOMEBRIDGE_OPTS
Restart=on-failure
RestartSec=10
KillMode=process

[Install]
WantedBy=multi-user.target" > /etc/systemd/system/homebridge.service

		useradd --system homebridge
	fi
	
	if [[ ! -d /var/homebridge ]]; then
		mkdir /var/homebridge
		cp /home/$MAINUSER/.homebridge/config.json /var/homebridge/
		cp -r /home/$MAINUSER/.homebridge/persist /var/homebridge
		chmod -R 0777 /var/homebridge
	fi
	
	## enable service
	systemctl -q is-active homebridge
	if [ $? -ne 0 ]; then
		systemctl daemon-reload
		systemctl enable homebridge
		systemctl start homebridge
	fi
}

# break loop on SIGUSR1
trap 'TIMEOUT=0' SIGUSR1

while [ 1 ]
do
	if [[ -z "$ZLLDB" ]] || [[ ! -f "$ZLLDB" ]]; then
		init
	fi

	while [[ $TIMEOUT -gt 0 ]]
	do
		sleep 1
		TIMEOUT=$((TIMEOUT - 1))
	done

	TIMEOUT=30

	[[ -z "$ZLLDB" ]] && continue
	[[ ! -f "$ZLLDB" ]] && continue

    checkHomebridge

done
